--------------------------------------------------------------------------------
-- Title   : Duplication block
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Signal duplication
--------------------------------------------------------------------------------
-- File     : MIK_duplication.vhd
-- Revision : x.x.x
-- Created  : April 28, 2015
-- Updated  : April 28, 2015
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2015 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity MIK_duplication is
  port (
    p_in      : in  std_logic;                     -- input
    p_outTTL  : out std_logic_vector (4 downto 0); -- TTL outputs
    p_outPECL : out std_logic_vector (4 downto 0)  -- LVPECL outputs
  );
end MIK_duplication;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
-- TTL duplication
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
architecture rtl_MIK_duplication of MIK_duplication is
  -- constant
  constant c_lenght_delayedOut4 : integer := 8;
  constant c_lenght_delayedOut3 : integer := 6;
  constant c_lenght_delayedOut2 : integer := 4;
  constant c_lenght_delayedOut1 : integer := 2;

  -- signal
  signal s_delayedOut4 : std_logic_vector (c_lenght_delayedOut4 downto 0);
  signal s_delayedOut3 : std_logic_vector (c_lenght_delayedOut3 downto 0);
  signal s_delayedOut2 : std_logic_vector (c_lenght_delayedOut2 downto 0);
  signal s_delayedOut1 : std_logic_vector (c_lenght_delayedOut1 downto 0);

  attribute keep : string;
  attribute keep of s_delayedOUT1 : signal is "True";
  attribute keep of s_delayedOUT2 : signal is "True";
  attribute keep of s_delayedOUT3 : signal is "True";
  attribute keep of s_delayedOUT4 : signal is "True";

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  s_delayedOut4(0) <= not p_in;
  genDelay4 : for i in 1 to (c_lenght_delayedOut4) generate
  begin
    s_delayedOut4(i) <=  not s_delayedOut4(i - 1);
  end generate genDelay4;
  p_outTTL(4) <= not s_delayedOut4(c_lenght_delayedOut4);

  s_delayedOut3(0) <= not p_in;
  genDelay3 : for i in 1 to (c_lenght_delayedOut3) generate
  begin
    s_delayedOut3(i) <=  not s_delayedOut3(i - 1);
  end generate genDelay3;
  p_outTTL(3) <= not s_delayedOut3(c_lenght_delayedOut3);

  s_delayedOut2(0) <= not p_in;
  genDelay2 : for i in 1 to (c_lenght_delayedOut2) generate
  begin
    s_delayedOut2(i) <=  not s_delayedOut2(i - 1);
  end generate genDelay2;
  p_outTTL(2) <= not s_delayedOut2(c_lenght_delayedOut2);

  s_delayedOut1(0) <= not p_in;
  genDelay1 : for i in 1 to (c_lenght_delayedOut1) generate
  begin
    s_delayedOut1(i) <=  not s_delayedOut1(i - 1);
  end generate genDelay1;
  p_outTTL(1) <= not s_delayedOut1(c_lenght_delayedOut1);

  -- Unused signals
  p_outTTL(0) <= '0'; -- the IO is used as input
  p_outPECL   <= "00000";

end rtl_MIK_duplication;
